'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const prefix = '@@redux-ops/';
const CREATE = `${prefix}CREATE`;
const UPDATE = `${prefix}UPDATE`;
const DELETE = `${prefix}DELETE`;
const CLEAR = `${prefix}CLEAR`;

const defaultState = {};
const createOperation = (state, action) => {
    const { payload: { id }, } = action;
    if (state[id]) {
        console.error(`ERROR: Cannot create new operation. Operation with id "${id}" already exists.`);
        return state;
    }
    return Object.assign({}, state, { [id]: Object.assign({}, action.payload) });
};
const updateOperation = (state, action) => {
    const { payload: { id }, } = action;
    if (!state[id]) {
        console.error(`ERROR: Cannot update operation. Operation with ${id} doesn't exist.`);
        return state;
    }
    return Object.assign({}, state, { [id]: Object.assign({}, action.payload) });
};
const deleteOperation = (state, action) => {
    const newState = Object.assign({}, state);
    delete newState[action.payload.id];
    return newState;
};
const clearOperations = (state, action) => (Object.assign({}, defaultState));
const handlers = {
    [CREATE]: createOperation,
    [UPDATE]: updateOperation,
    [DELETE]: deleteOperation,
    [CLEAR]: clearOperations,
};
function reducer(state = defaultState, action) {
    const handler = handlers[action.type];
    return handler ? handler(state, action) : state;
}

function buildOperation(id, status, data) {
    return { id, status: status || "default", data };
}

const createOperation$1 = (id, status, data) => ({
    type: CREATE,
    payload: buildOperation(id, status, data),
});
const updateOperation$1 = (id, status, data) => ({
    type: UPDATE,
    payload: buildOperation(id, status, data),
});
const deleteOperation$1 = (id) => ({
    type: DELETE,
    payload: { id },
});
const clearOperations$1 = () => ({
    type: CLEAR,
});

let reducerName = 'ops';
function setReducerName(name) {
    reducerName = name;
}
function getOpsState(state) {
    const subState = state[reducerName];
    if (subState === undefined) {
        throw new Error(`No reducer found with name ${reducerName}. Try to set it via setReducerName().`);
    }
    return subState;
}
function getOps(state) {
    return getOpsState(state);
}
function getOpById(state, id) {
    return getOps(state)[id];
}
function searchOps(state, query) {
    const ops = getOps(state);
    return Object.keys(ops)
        .filter(id => !!id.match(query))
        .map(id => ops[id]);
}

exports.default = reducer;
exports.defaultState = defaultState;
exports.prefix = prefix;
exports.CREATE = CREATE;
exports.UPDATE = UPDATE;
exports.DELETE = DELETE;
exports.CLEAR = CLEAR;
exports.createOperation = createOperation$1;
exports.updateOperation = updateOperation$1;
exports.deleteOperation = deleteOperation$1;
exports.clearOperations = clearOperations$1;
exports.setReducerName = setReducerName;
exports.getOpsState = getOpsState;
exports.getOps = getOps;
exports.getOpById = getOpById;
exports.searchOps = searchOps;
